#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing proposition information scraped from the CAL-ACCESS website.
"""
from __future__ import unicode_literals
from django.db import models
from calaccess_scraped.models.base import BaseScrapedModel


class Proposition(BaseScrapedModel):
    """
    A yes or no ballot measure for voters scraped from the California Secretary of State's site.
    """
    # Most of the time, this is a number, however,
    # it can be a bona fide name, e.g.
    # '2003 Recall Question'
    name = models.CharField(
        verbose_name="proposition name",
        max_length=200
    )
    scraped_id = models.CharField(
        verbose_name="proposition identification number",
        max_length=200
    )
    election = models.ForeignKey(
        'PropositionElection',
        related_name='propositions',
        on_delete=models.CASCADE
    )

    class Meta:
        """
        Model options.
        """
        ordering = ("-election", "name")

    def __str__(self):
        return 'Proposition: {}'.format(self.name)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-26 14:45
from __future__ import unicode_literals

import datetime
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CandidateScrapedElection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField(max_length=2000, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=200)),
                ('scraped_id', models.CharField(blank=True, max_length=3, verbose_name='election identification number')),
                ('sort_index', models.IntegerField(help_text='The index value is used to preserve sorting of elections, since multiple elections may occur in a year. A greater sort index corresponds to a more recent election.', null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='IncumbentScrapedElection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField(max_length=2000, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=200)),
                ('session', models.IntegerField(verbose_name='session start year')),
                ('date', models.DateField(verbose_name='election date')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PropositionScrapedElection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField(max_length=2000, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=200)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ScrapedCandidate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField(max_length=2000, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=200, verbose_name='candidate name')),
                ('scraped_id', models.CharField(blank=True, max_length=7, verbose_name='candidate identification number')),
                ('office_name', models.CharField(blank=True, max_length=100, verbose_name='name of the office for which this candidate is running')),
                ('election', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='candidates', to='calaccess_scraped.CandidateScrapedElection')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ScrapedCandidateCommittee',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField(max_length=2000, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=500, verbose_name='committee name')),
                ('scraped_id', models.CharField(max_length=7, verbose_name='committee identification number')),
                ('candidate_id', models.CharField(max_length=100)),
                ('status', models.CharField(max_length=100)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ScrapedDataVersion',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('process_start_datetime', models.DateTimeField(default=datetime.datetime.now, help_text='Date and time when the scraping started', null=True, verbose_name='date and time processing started')),
                ('process_finish_datetime', models.DateTimeField(help_text='Date and time when the scraping finished', null=True, verbose_name='date and time update finished')),
            ],
            options={
                'ordering': ('-process_start_datetime',),
                'get_latest_by': 'process_start_datetime',
                'verbose_name': 'TRACKING: CAL-ACCESS scraped data version',
            },
        ),
        migrations.CreateModel(
            name='ScrapedIncumbent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField(max_length=2000, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('session', models.IntegerField(verbose_name='session start year')),
                ('category', models.CharField(max_length=100, verbose_name='office category')),
                ('office_name', models.CharField(max_length=100, verbose_name='name of the office')),
                ('name', models.CharField(max_length=200, verbose_name='name of the incumbent')),
                ('scraped_id', models.CharField(max_length=7, verbose_name='scraped identification number')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ScrapedProposition',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField(max_length=2000, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=200, verbose_name='proposition name')),
                ('scraped_id', models.CharField(max_length=200, verbose_name='proposition identification number')),
                ('election', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='propositions', to='calaccess_scraped.PropositionScrapedElection')),
            ],
            options={
                'ordering': ('-election', 'name'),
            },
        ),
        migrations.CreateModel(
            name='ScrapedPropositionCommittee',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField(max_length=2000, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=500, verbose_name='committee name')),
                ('scraped_id', models.CharField(max_length=7, verbose_name='committee identification number')),
                ('position', models.CharField(help_text='Whether the committee supports or opposes the proposition', max_length=100)),
                ('proposition', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='calaccess_scraped.ScrapedProposition')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing election information scraped from the CAL-ACCESS website.
"""
from __future__ import unicode_literals
from django.db import models
from calaccess_scraped.models.base import BaseScrapedElection


class CandidateElection(BaseScrapedElection):
    """
    An election day scraped as part of the `scrapecalaccesscandidates` command.
    """
    scraped_id = models.CharField(
        verbose_name="election identification number",
        max_length=3,
        blank=True,
    )
    sort_index = models.IntegerField(
        null=True,
        help_text="The index value is used to preserve sorting of elections, "
                  "since multiple elections may occur in a year. A greater sort "
                  "index corresponds to a more recent election."
    )

    def __str__(self):
        return self.name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing incumbent information scraped from the CAL-ACCESS website.
"""
from __future__ import unicode_literals
from django.db import models
from .base import (
    BaseScrapedModel,
    BaseScrapedElection,
)


class Incumbent(BaseScrapedModel):
    """
    An incumbent state official scraped from the California Secretary of State's site.
    """
    session = models.IntegerField(
        verbose_name="session start year",
    )
    category = models.CharField(
        verbose_name="office category",
        max_length=100,
    )
    office_name = models.CharField(
        verbose_name="name of the office",
        max_length=100,
    )
    name = models.CharField(
        verbose_name="name of the incumbent",
        max_length=200,
    )
    scraped_id = models.CharField(
        verbose_name="scraped identification number",
        max_length=7,
    )

    def __str__(self):
        return self.name


class IncumbentElection(BaseScrapedElection):
    """
    An election day scraped as part of the `scrapecalaccessincumbents` command.
    """
    session = models.IntegerField(
        verbose_name="session start year",
    )
    date = models.DateField(
        verbose_name="election date",
    )

    def __str__(self):
        return self.name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Base models for storing information scraped from the CAL-ACCESS website.
"""
from __future__ import unicode_literals
import textwrap
from django.db import models
from django.db.models.base import ModelBase
from django.template.defaultfilters import capfirst


class CalAccessMetaClass(ModelBase):
    """
    A custom metaclass for our base model.

    Automatically configures Meta attributes common to all models.
    """
    def __new__(cls, name, bases, attrs):
        """
        Override the default __new__ behavior.
        """
        klass = super(CalAccessMetaClass, cls).__new__(cls, name, bases, attrs)

        # Cook up an automated verbose name for each model
        klass_str = str(klass).replace("<class 'calaccess_scraped.models.", "")
        klass_group = klass_str.split(".")[0].upper()
        klass_table = capfirst(klass._meta.verbose_name_plural)
        klass_name = "{0}: {1}".format(klass_group, klass_table)

        # Insert the verbose name into each model's configuration
        klass._meta.verbose_name = klass_name
        klass._meta.verbose_name_plural = klass_name

        # Set the app_label too
        klass._meta.app_label = "calaccess_scraped"

        # Finish up
        return klass


class CalAccessBaseModel(models.Model):
    """
    An abstract model with some tricks we'll reuse.
    """
    __metaclass__ = CalAccessMetaClass

    def doc(self):
        """
        Return the model's docstring as a readable string ready to print.
        """
        if self.__doc__.startswith(self.klass_name):
            return ''
        return textwrap.dedent(self.__doc__).strip()

    @property
    def db_table(self):
        """
        Return the model's database table name as a string.
        """
        return self._meta.db_table

    @property
    def klass(self):
        """
        Return the model class itself.
        """
        return self.__class__

    @property
    def klass_name(self):
        """
        Return the name of the model class.
        """
        return self.__class__.__name__

    @property
    def klass_group(self):
        """
        Return the name of the model's group, as determined by its submodule.
        """
        return str(self.__class__).split(".")[-2]

    def get_field_list(self):
        """
        Return all the fields on the model as a list.
        """
        return self._meta.fields

    class Meta:
        """
        Meta model options.
        """
        abstract = True
        app_label = 'calaccess_scraped'


class BaseScrapedModel(CalAccessBaseModel):
    """
    Abstract base model from which all scraped models inherit.
    """
    url = models.URLField(max_length=2000, null=True)
    created = models.DateTimeField(auto_now_add=True)
    last_modified = models.DateTimeField(auto_now=True)

    class Meta:
        """
        Model options.
        """
        abstract = True


class BaseScrapedElection(BaseScrapedModel):
    """
    An election day scraped from the California Secretary of State's site.

    This is an abstract base model that creates two tables, one for elections
    scraped as part of the candidates scraper, and one for elections scraped
    as part of the propositions scraper.
    """
    name = models.CharField(max_length=200)

    class Meta:
        """
        Model options.
        """
        abstract = True


class BaseScrapedCommittee(BaseScrapedModel):
    """
    An committee scraped from the California Secretary of State's site.

    This is an abstract base model that creates two tables, one for committees
    scraped as part of the candidates scraper, and one for committees scraped
    as part of the propositions scraper.
    """
    name = models.CharField(
        verbose_name="committee name",
        max_length=500
    )
    scraped_id = models.CharField(
        verbose_name="committee identification number",
        max_length=7
    )

    class Meta:
        """
        Model options.
        """
        abstract = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Run all scraper commands.
"""
# Files
import os
import csv
from calaccess_scraped import get_data_directory

# Time
from django.utils.timezone import now

# Django
from calaccess_scraped import models
from calaccess_scraped.management.commands import CalAccessCommand


class Command(CalAccessCommand):
    """
    Run all scraper commands.
    """
    help = "Scrape CAL-ACCESS data and sync it with the database"
    data_dir = get_data_directory()

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        parser.add_argument(
            '--flush',
            action='store_true',
            dest='force_flush',
            default=False,
            help='Flush database tables'
        )

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # Parse the arguments
        self.force_flush = options.get("force_flush")

        # Flush, if asked for.
        if self.force_flush:
            self.flush()

        # Sync the scrape with the database.
        self.sync()

    def flush(self):
        """
        Delete records form related database tables.
        """
        self.log("Flushing database")
        models.PropositionCommittee.objects.all().delete()
        models.Proposition.objects.all().delete()
        models.PropositionElection.objects.all().delete()
        models.Candidate.objects.all().delete()
        models.CandidateElection.objects.all().delete()
        models.Incumbent.objects.all().delete()
        models.IncumbentElection.objects.all().delete()

    def open_csv(self, model):
        """
        Returns a DictReader with all the scraped records for the provided model.
        """
        file_name = "{}Item.csv".format(model().klass_name)
        file_path = os.path.join(self.data_dir, file_name)
        with open(file_path, 'r') as file_obj:
            file_reader = list(csv.DictReader(file_obj))
            if self.verbosity:
                self.log("Syncing {} {} scraped items".format(len(file_reader), model().klass_name))
            return file_reader

    def save_row(self, model, **row):
        """
        Syncs a CSV row of data to the provided model.
        """
        obj, created = model.objects.get_or_create(**row)
        if created and self.verbosity > 2:
            self.log('Created {}: {}'.format(obj.klass_name, obj))
        if not created:
            obj.last_modified = now()
            obj.save()

    def sync(self):
        """
        Syncs the scraped data with the database, model by model and row by row.
        """
        # Load the models one by one.
        for d in self.open_csv(models.PropositionElection):
            self.save_row(models.PropositionElection, name=d['name'], url=d['url'])

        for d in self.open_csv(models.Proposition):
            self.save_row(
                models.Proposition,
                name=d['name'],
                scraped_id=d['id'],
                url=d['url'],
                election=models.PropositionElection.objects.get(name=d['election_name'])
            )

        for d in self.open_csv(models.PropositionCommittee):
            self.save_row(
                models.PropositionCommittee,
                name=d['name'],
                scraped_id=d['id'],
                position=d['position'],
                url=d['url'],
                proposition=models.Proposition.objects.get(name=d['proposition_name'])
            )

        for d in self.open_csv(models.IncumbentElection):
            self.save_row(
                models.IncumbentElection,
                session=d['session'],
                name=d['name'],
                date=d['date'],
                url=d['url'],
            )

        for d in self.open_csv(models.Incumbent):
            self.save_row(
                models.Incumbent,
                session=d['session'],
                category=d['category'],
                office_name=d['office'],
                name=d['name'],
                url=d['url'],
                scraped_id=d['id']
            )

        for d in self.open_csv(models.CandidateElection):
            self.save_row(
                models.CandidateElection,
                name=d['name'],
                scraped_id=d['id'],
                url=d['url'],
            )

        for d in self.open_csv(models.Candidate):
            self.save_row(
                models.Candidate,
                name=d['name'],
                scraped_id=d['id'],
                office_name=d['office'],
                url=d['url'],
                election=models.CandidateElection.objects.get(scraped_id=d['election_id']),
            )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing committee information scraped from the CAL-ACCESS website.
"""
from __future__ import unicode_literals
from django.db import models
from calaccess_scraped.models.base import BaseScrapedCommittee


class CandidateCommittee(BaseScrapedCommittee):
    """
    A candidate committee scraped from the California Secretary of State's site.
    """
    candidate_id = models.CharField(max_length=100)
    status = models.CharField(max_length=100)

    def __str__(self):
        return self.name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom administration panels for scraped candidate models.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_scraped import models
from calaccess_scraped.admin.base import BaseAdmin


@admin.register(models.IncumbentElection)
class IncumbentElectionAdmin(BaseAdmin):
    """
    Custom admin for IncumbentElectionAdmin model.
    """
    list_display = (
        "name",
        "date"
    )
    list_per_page = 500
    search_fields = ("name",)


@admin.register(models.Incumbent)
class IncumbentAdmin(BaseAdmin):
    """
    Custom admin for IncumbentAdmin model.
    """
    list_display = (
        "scraped_id",
        "name",
        "category",
        "office_name",
        "session"
    )
    list_filter = (
        "category",
        "office_name",
        "session"
    )
    list_per_page = 500
    search_fields = (
        "scraped_id",
        "name",
        "category",
        "office_name"
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing committee information scraped from the CAL-ACCESS website.
"""
from __future__ import unicode_literals
from django.db import models
from calaccess_scraped.models.base import BaseScrapedCommittee


class PropositionCommittee(BaseScrapedCommittee):
    """
    A committee supporting or opposing a proposition scraped from the California Secretary of State's site.
    """
    position = models.CharField(
        max_length=100,
        help_text="Whether the committee supports or opposes the proposition",
    )
    proposition = models.ForeignKey('Proposition', on_delete=models.CASCADE)

    def __str__(self):
        return self.name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom administration panels for scraped proposition models.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_scraped import models
from calaccess_scraped.admin.base import BaseAdmin


@admin.register(models.PropositionElection)
class PropositionElectionAdmin(BaseAdmin):
    """
    Custom admin for PropositionElectionAdmin model.
    """
    list_display = ("name",)
    list_per_page = 500
    search_fields = ("name",)


@admin.register(models.Proposition)
class PropositionAdmin(BaseAdmin):
    """
    Custom admin for ScrapedPropositionAdmin model.
    """
    list_display = (
        "scraped_id",
        "name",
        "election"
    )
    list_filter = ("election",)
    list_per_page = 500
    search_fields = (
        "name",
        "scraped_id"
    )


@admin.register(models.PropositionCommittee)
class PropositionCommitteeAdmin(BaseAdmin):
    """
    Custom admin for PropositionCommitteeAdmin model.
    """
    list_display = (
        "scraped_id",
        "name",
        "position",
        "proposition"
    )
    list_per_page = 500
    search_fields = (
        "scraped_id",
        "name",
        "position",
        "proposition"
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing proposition information scraped from the CAL-ACCESS website.
"""
from .propositions import Proposition
from .elections import PropositionElection
from .committees import PropositionCommittee


__all__ = (
    'Proposition',
    'PropositionElection',
    'PropositionCommittee',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Unittests for management commands.
"""
# from django.core.management import call_command
from django.test import TestCase
from calaccess_scraped.management.commands import CalAccessCommand


class ScrapedDataCommandsTest(TestCase):
    """
    Run and test management commands.
    """
    fixtures = [
        'calaccess_scraped/fixtures/candidate_election.json',
        'calaccess_scraped/fixtures/candidate.json',
        'calaccess_scraped/fixtures/incumbent_election.json',
        'calaccess_scraped/fixtures/incumbent.json',
        'calaccess_scraped/fixtures/proposition_election.json',
        'calaccess_scraped/fixtures/proposition.json',
    ]

    def test_commands(self):
        """
        Run the scraping commands.
        """
        pass

    def test_base_command(self):
        """
        Test options on base commands.
        """
        c = CalAccessCommand()
        c.handle()
        c.header("")
        c.log("")
        c.success("")
        c.warn("")
        c.failure("")
        c.duration()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Base classes for custom management commands.
"""
import re
import logging
from django.utils import timezone
from django.utils.termcolors import colorize
from django.core.management.base import BaseCommand
logger = logging.getLogger(__name__)


class CalAccessCommand(BaseCommand):
    """
    Base class for all custom CalAccess-related management commands.
    """
    def handle(self, *args, **options):
        """
        Sets options common to all commands.

        Any command subclassing this object should implement its own
        handle method, as is standard in Django, and run this method
        via a super call to inherit its functionality.
        """
        # Set global options
        self.verbosity = options.get("verbosity")
        self.no_color = options.get("no_color")

        # Start the clock
        self.start_datetime = timezone.now()

    def header(self, string):
        """
        Writes out a string to stdout formatted to look like a header.
        """
        logger.debug(string)
        if not getattr(self, 'no_color', None):
            string = colorize(string, fg="cyan", opts=("bold",))
        self.stdout.write(string)

    def log(self, string):
        """
        Writes out a string to stdout formatted to look like a standard line.
        """
        logger.debug(string)
        if not getattr(self, 'no_color', None):
            string = colorize("%s" % string, fg="white")
        self.stdout.write(string)

    def success(self, string):
        """
        Writes out a string to stdout formatted green to communicate success.
        """
        logger.debug(string)
        if not getattr(self, 'no_color', None):
            string = colorize(string, fg="green")
        self.stdout.write(string)

    def warn(self, string):
        """
        Writes string to stdout formatted yellow to communicate a warning.
        """
        logger.warn(string)
        if not getattr(self, 'no_color', None):
            string = colorize(string, fg="yellow")
        self.stdout.write(string)

    def failure(self, string):
        """
        Writes string to stdout formatted red to communicate failure.
        """
        logger.error(string)
        if not getattr(self, 'no_color', None):
            string = colorize(string, fg="red")
        self.stdout.write(string)

    def duration(self):
        """
        Calculates how long command has been running and writes it to stdout.
        """
        duration = timezone.now() - self.start_datetime
        self.stdout.write('Duration: {}'.format(str(duration)))
        logger.debug('Duration: {}'.format(str(duration)))

    def __str__(self):
        return re.sub(r'(.+\.)*', '', self.__class__.__module__)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import all of the admins from submodules and thread them together.
"""
from .base import BaseAdmin
from .candidates import (
    CandidateElectionAdmin,
    CandidateAdmin,
    CandidateCommitteeAdmin,
)
from .incumbents import (
    IncumbentElectionAdmin,
    IncumbentAdmin,
)
from .propositions import (
    PropositionElectionAdmin,
    PropositionAdmin,
    PropositionCommitteeAdmin,
)
from .tracking import (
    ScrapedDataVersionAdmin,
)

__all__ = (
    'BaseAdmin',
    'ScrapedDataVersionAdmin',
    'CandidateElectionAdmin',
    'CandidateAdmin',
    'CandidateCommitteeAdmin',
    'IncumbentElectionAdmin',
    'IncumbentAdmin',
    'PropositionElectionAdmin',
    'PropositionAdmin',
    'PropositionCommitteeAdmin',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing election information scraped from the CAL-ACCESS website.
"""
from __future__ import unicode_literals
from calaccess_scraped.models.base import BaseScrapedElection


class PropositionElection(BaseScrapedElection):
    """
    An election day scraped as part of the `scrapecalaccesspropositions` command.
    """
    def __str__(self):
        return self.name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing candidate information scraped from the CAL-ACCESS website.
"""
from .candidates import Candidate
from .elections import CandidateElection
from .committees import CandidateCommittee

__all__ = (
    'Candidate',
    'CandidateElection',
    'CandidateCommittee',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing candidate information scraped from the CAL-ACCESS website.
"""
from __future__ import unicode_literals
from django.db import models
from calaccess_scraped.models.base import BaseScrapedModel


class Candidate(BaseScrapedModel):
    """
    A candidate for office scraped from the California Secretary of State's site.
    """
    name = models.CharField(
        verbose_name="candidate name",
        max_length=200
    )
    scraped_id = models.CharField(
        verbose_name="candidate identification number",
        max_length=7,
        blank=True,  # Some don't have IDs on the website
    )
    office_name = models.CharField(
        verbose_name="name of the office for which this candidate is running",
        max_length=100,
        blank=True
    )
    election = models.ForeignKey(
        'CandidateElection',
        related_name='candidates',
        on_delete=models.CASCADE
    )

    def __str__(self):
        return self.name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from distutils.core import Command
from setuptools import setup, find_packages


def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as f:
        return f.read()


class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from django.conf import settings
        settings.configure(
            DATABASES={
                'default': {
                    'NAME': ':memory:',
                    'ENGINE': 'django.db.backends.sqlite3'
                }
            },
            INSTALLED_APPS=('calaccess_scraped',),
            MIDDLEWARE_CLASSES=()
        )
        from django.core.management import call_command
        import django
        django.setup()
        call_command('test', 'calaccess_scraped')


setup(
    name='django-calaccess-scraped-data',
    version='3.0.0',
    author='California Civic Data Coalition',
    author_email='cacivicdata@gmail.com',
    url='http://django-calaccess.californiacivicdata.org',
    description='A Django app to scrape campaign-finance data from '
                'the California Secretary of State’s CAL-ACCESS website',
    long_description=read('README.rst'),
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,  # because we're including static files
    cmdclass={'test': TestCommand},
    install_requires=(
        'django>=3.2.*',
        'pytz'
    ),
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'License :: OSI Approved :: MIT License'
    ),
    project_urls={
        'Project': 'https://www.californiacivicdata.org/',
        'Documentation': 'https://django-calaccess.californiacivicdata.org',
        'Funding': 'https://www.californiacivicdata.org/about/',
        'Source': 'https://github.com/california-civic-data-coalition/django-calaccess-scraped-data',
        'Testing': 'https://github.com/california-civic-data-coalition/django-calaccess-scraped-data/actions/workflows/tests.yaml',
        'Tracker': 'https://github.com/california-civic-data-coalition/django-calaccess-scraped-data/issues'
    },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom administration panels for tracking models.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_scraped import models
from calaccess_scraped.admin.base import BaseAdmin


@admin.register(models.ScrapedDataVersion)
class ScrapedDataVersionAdmin(BaseAdmin):
    """
    Custom admin for the ScrapedDataVersion model.
    """
    list_display = (
        "id",
        "process_start_datetime",
        "process_finish_datetime"
    )
    list_display_links = ('process_start_datetime',)
    list_filter = ("process_finish_datetime",)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-28 15:00
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_scraped', '0001_initial'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='ScrapedCandidate',
            new_name='Candidate',
        ),
        migrations.RenameModel(
            old_name='ScrapedCandidateCommittee',
            new_name='CandidateCommittee',
        ),
        migrations.RenameModel(
            old_name='CandidateScrapedElection',
            new_name='CandidateElection',
        ),
        migrations.RenameModel(
            old_name='ScrapedIncumbent',
            new_name='Incumbent',
        ),
        migrations.RenameModel(
            old_name='IncumbentScrapedElection',
            new_name='IncumbentElection',
        ),
        migrations.RenameModel(
            old_name='ScrapedProposition',
            new_name='Proposition',
        ),
        migrations.RenameModel(
            old_name='ScrapedPropositionCommittee',
            new_name='PropositionCommittee',
        ),
        migrations.RenameModel(
            old_name='PropositionScrapedElection',
            new_name='PropositionElection',
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import all of the models from submodules and thread them together.
"""
from .base import (
    CalAccessMetaClass,
    CalAccessBaseModel,
    BaseScrapedModel,
    BaseScrapedElection,
    BaseScrapedCommittee,
)
from .candidates import (
    Candidate,
    CandidateElection,
    CandidateCommittee,
)
from .incumbents import (
    Incumbent,
    IncumbentElection,
)
from .propositions import (
    Proposition,
    PropositionElection,
    PropositionCommittee,
)
from .tracking import ScrapedDataVersion

__all__ = (
    'CalAccessMetaClass',
    'CalAccessBaseModel',
    'BaseScrapedModel',
    'BaseScrapedElection',
    'BaseScrapedCommittee',
    'Proposition',
    'PropositionElection',
    'PropositionCommittee',
    'Candidate',
    'CandidateElection',
    'CandidateCommittee',
    'Incumbent',
    'IncumbentElection',
    'ScrapedDataVersion',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Basic configuration for the application.
"""
from django.apps import AppConfig


class CalAccessScrapedConfig(AppConfig):
    """
    Application configuration.
    """
    name = 'calaccess_scraped'
    verbose_name = "CAL-ACCESS scraped data"
    default_auto_field = 'django.db.models.BigAutoField'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for tracking processing of CAL-ACCESS snapshots over time.
"""
from __future__ import unicode_literals
from django.db import models
from datetime import datetime


class ScrapedDataVersion(models.Model):
    """
    A version of CAL-ACCESS scraped data.
    """
    process_start_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time processing started',
        help_text='Date and time when the scraping started',
        default=datetime.now,
    )
    process_finish_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time update finished',
        help_text='Date and time when the scraping finished'
    )

    class Meta:
        """
        Meta model options.
        """
        app_label = 'calaccess_scraped'
        verbose_name = 'TRACKING: CAL-ACCESS scraped data version'
        ordering = ('-process_start_datetime',)
        get_latest_by = 'process_start_datetime'

    def __str__(self):
        return str(self.process_start_datetime)

    @property
    def update_completed(self):
        """
        Check if the database update to the version completed.

        Return True or False.
        """
        if self.process_finish_datetime:
            is_completed = True
        else:
            is_completed = False

        return is_completed

    @property
    def update_stalled(self):
        """
        Check if the database update to the version started but did not complete.

        Return True or False.
        """
        if self.process_start_datetime and not self.update_finish_datetime:
            is_stalled = True
        else:
            is_stalled = False

        return is_stalled



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
General utilities for the application.
"""
import os
default_app_config = 'calaccess_scraped.apps.CalAccessScrapedConfig'


def get_data_directory():
    """
    Returns directory for scraped data.
    """
    return os.path.join(os.path.dirname(__file__), 'data')
